#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Contains methods for retrieving embedded data from the nmdc-schema library."""

import io
import json
import pkgutil
from os.path import getatime
from typing import Dict, List

import click
import yaml
from linkml.utils.rawloader import load_raw_schema
from linkml_runtime.linkml_model.meta import SchemaDefinition
from linkml_runtime.utils.schemaview import SchemaView


def get_nmdc_yaml_bytesIO() -> io.BytesIO:
    """Returns the nmdc.yaml file as bytes steam.
    This function is not intended to be used directly, but it used by other functions

    Returns
    -------
    BytesIO
        A bytes stream of nmdc.yaml file.
    """
    # get nmdc.yaml file from the package data
    return io.BytesIO(pkgutil.get_data(__name__, "nmdc_schema_merged.yaml"))


def get_nmdc_yaml_bytes() -> bytes:
    """Retruns the nmdc.yaml file as bytes.

    Returns
    -------
    bytes
        The bytes of thenmdc.yaml file.
    """
    nmdc_yaml = get_nmdc_yaml_bytesIO()
    return nmdc_yaml.getvalue()


def get_nmdc_yaml_string() -> str:
    """Retruns the nmdc.yaml file as a string.

    Returns
    -------
    str
        A string containing the contents of nmdc.yaml file.
    """
    nmdc_yaml = get_nmdc_yaml_bytes()
    return nmdc_yaml.decode("utf-8")


def get_materialized_nmdc_yaml_string():
    materialized_nmdc_bytes_io = io.BytesIO(pkgutil.get_data(__name__, "nmdc_materialized_patterns.yaml"))
    materialized_nmdc_yaml_bytes = materialized_nmdc_bytes_io.getvalue()
    materialized_nmdc_yaml_string = materialized_nmdc_yaml_bytes.decode("utf-8")
    return materialized_nmdc_yaml_string


def get_nmdc_jsonschema_bytesIO() -> io.BytesIO:
    """Returns the nmdc.schema.json file as bytes steam.
    This function is not intended to be used directly, but it used by other functions

    Returns
    -------
    BytesIO
        A bytes stream of nmdc.schema.json file.
    """
    # get nmdc.yaml file from the package data
    return io.BytesIO(pkgutil.get_data(__name__, "nmdc.schema.json"))


def get_nmdc_jsonschema_bytes() -> bytes:
    """Retruns the nmdc.schema.json file as bytes.

    Returns
    -------
    bytes
        The bytes of the nmdc.schema.json file.
    """
    nmdc_json = get_nmdc_jsonschema_bytesIO()
    return nmdc_json.getvalue()


def get_nmdc_jsonschema_string() -> str:
    """Retruns the nmdc.schema.json file as a string.

    Returns
    -------
    str
        A string containing the contents of nmdc.schema.json file.
    """
    nmdc_json = get_nmdc_jsonschema_bytes()
    return nmdc_json.decode("utf-8")


def get_nmdc_jsonschema_dict() -> Dict:
    """Parses the nmdc.schema.json file into a dict.

    Returns
    -------
    dict
        The dict of the keys and value in the nmdc.schema.json file.
    """
    nmdc_json = get_nmdc_jsonschema_bytes()
    return json.loads(nmdc_json)


def get_nmdc_jsonschema() -> str:
    """
    Returns the NMDC jsonschema (nmdc.schema.json) as json.

    Returns
    -------
    str
        JSON string representation of the NMDC jsonschema (nmdc.schema.json).
    """
    nmdc_schema = get_nmdc_jsonschema_dict()
    return json.dumps(nmdc_schema, indent=2)


def get_nmdc_schema_definition() -> SchemaDefinition:
    """Returns a LinkML SchemaDefintion object created from the nmdc.yaml file.

    Returns
    -------
    SchemaDefinition
        A SchemaDefintion object created from nmdc.yaml file.
    """
    nmdc_yaml = get_nmdc_yaml_string()
    return load_raw_schema(nmdc_yaml)


def get_nmdc_file_type_enums() -> List[Dict[str, str]]:
    """Returns list of dicts with information about each NMDC file enums.
    Each dict in the list has the key/values:
    {
        name: the name of the enum
        description: the description of the enum
        file_name_pattern: the pattern for the file type or None if not present
    }

    Returns
    -------
    List[Dict[str, str]]
    List of dicts with information about each NMDC file enum.
    """
    schema = get_nmdc_schema_definition()
    file_enums = schema.enums["file type enum"].permissible_values  # returns a dict

    # todo: report error
    # view = SchemaView(schema)
    # file_enums = view.get_enum("file type enum").permissible_values  # returns a dict

    # create list of objects with info about each file type enum
    enum_objs = []
    for key, permissible_val in file_enums.items():
        obj = {"name": key}
        # obj = {"name": permissible_val.text}
        obj["description"] = permissible_val.description

        # not every file type enum will have a file name pattern
        file_name_pattern = permissible_val.annotations.get("file_name_pattern", None)
        if file_name_pattern:
            obj["file_name_pattern"] = file_name_pattern.value
        else:
            obj["file_name_pattern"] = None

        enum_objs.append(obj)

    return enum_objs


def get_nmdc_file_type_enums_json() -> str:
    """Returns informaton about the file type enums as json.
    Each object contains the following key/values:
    {
        name: the name of the enum
        description: the description of the enum
        file_name_pattern: the pattern for the file type or null if not present
    }
    Note: Calls get_nmdc_file_type_enums() to fetch information.

    Returns
    -------
    str
        JSON formated string of file type enum information.getatime
    """
    file_enums = get_nmdc_file_type_enums()
    return json.dumps(file_enums, indent=2)


def get_gold_sssom() -> str:
    """Returns the gold-to-mixs.sssom.tsv file from package.

    Returns
    -------
    str
        the gold-to-mixs.sssom.tsv file
    """
    sssom = io.BytesIO(pkgutil.get_data(__name__, "gold-to-mixs.sssom.tsv"))
    return sssom.getvalue().decode("utf-8")


##### CLI interface #####
@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option(
    "--fetch",
    "-f",
    default="",
    help="""\b
    Fetches the specified data file from the nmdc-schema library.
    Only one argument is permitted.
    \b
    fetch arguments:
    yaml            returns the merged nmdc.yaml file as a string
    jsonschema      returns the NMDC jsonschema as json
    dict            returns the NMDC jsonschema as a dict
    schemadef       returns the LinkML SchemaDefintion created from the merged nmdc.yaml file
    filetypeenums   returns informaton about the NMDC file type enums as json
    goldsssom       returns the gold-to-mixs.sssom.tsv file contents
    """,
)
@click.pass_context
# def cli(ctx, as_yaml, as_json, as_bytes):
def cli(ctx, fetch):
    if "yaml" == fetch:
        click.echo(get_nmdc_yaml_string())
    elif "bytes" == fetch:
        click.echo(get_nmdc_yaml_bytes())
    elif "jsonschema" == fetch:
        click.echo(get_nmdc_jsonschema())
    elif "dict" == fetch:
        click.echo(get_nmdc_jsonschema_dict())
    elif "schemadef" == fetch:
        click.echo(get_nmdc_schema_definition())
    elif "filetypeenums" == fetch:
        click.echo(get_nmdc_file_type_enums_json())
    elif "goldsssom" == fetch:
        click.echo(get_gold_sssom())
    else:
        click.echo(ctx.get_help())


if __name__ == "__main__":
    cli()  # CLI interface



import pprint
from typing import List

import click

from nmdc_schema.get_nmdc_view import ViewGetter


class ClassSlotsGetter:
    @staticmethod
    def get_class_slots(class_name) -> List[str]:
        # assumes user wants slots from induced class
        # if the user adds a MIxS slot to class Biosample, will the view be created?
        # might need a pure YAML solution
        # or user could just add slot name to ???
        # most importantly, this get the view from nmdc_schema,nmdc_schema_merged.yaml
        # which won't include the user's changes!!!
        view_getter = ViewGetter()
        nmdc_view = view_getter.get_view()
        nmdc_class = nmdc_view.induced_class(class_name)
        class_slots = nmdc_class.attributes

        class_slot_names = [v.name for k, v in class_slots.items()]
        class_slot_names.sort()
        return class_slot_names


@click.command()
@click.option('--output_file',
              '-o',
              type=click.File('w'),
              required=True,
              help='Your output will be written as a single-column TSV file.')
@click.option('--class_name',
              '-c',
              required=True,
              help='For what class do you want to retrieve slots?.')
def main(output_file, class_name):
    class_slot_getter = ClassSlotsGetter()
    class_used_slots = class_slot_getter.get_class_slots(class_name)

    with output_file:
        for slot in class_used_slots:
            output_file.write(f"{slot}\n")

    click.echo(f"The result has been written to {output_file}")


if __name__ == '__main__':
    main()



import nltk
from nltk.util import ngrams
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity

list_a = ['barren land', 'cropland', 'deciduous forest', 'developed open space',
          'developed space with high usage intensity', 'developed space with low usage intensity',
          'developed space with medium usage intensity', 'dwarf scrub', 'emergent herbaceous wetland',
          'evergreen forest', 'gramanoid or herbaceous vegetation', 'lichen-dominated vegetation', 'mixed forest',
          'moss-dominated vegetation', 'open water', 'pastureland or hayfields', 'perennial ice or snow', 'scrub',
          'sedge- and forb-dominated herbaceous vegetation', 'woody wetland']
list_b = ['biome', 'terrestrial biome', 'marine biome', 'freshwater biome', 'large river biome',
          'large river headwater biome', 'large river delta biome', 'small river biome', 'large freshwater lake biome',
          'small freshwater lake biome', 'xeric basin biome', 'aquatic biome',
          'concentration basin mediterranean sea biome', 'estuarine biome', 'marine salt marsh biome',
          'marine pelagic biome', 'marine benthic biome', 'marine neritic benthic zone biome',
          'marine bathyal zone biome', 'marine abyssal zone biome', 'marine hadal zone biome', 'marine reef biome',
          'marine hydrothermal vent biome', 'neritic pelagic zone biome', 'oceanic pelagic zone biome',
          'oceanic sea surface microlayer biome', 'oceanic epipelagic zone biome', 'oceanic mesopelagic zone biome',
          'oceanic bathypelagic zone biome', 'oceanic abyssopelagic zone biome', 'oceanic hadal pelagic zone biome',
          'oceanic benthopelagic zone biome', 'neritic sea surface microlayer biome', 'neritic epipelagic zone biome',
          'neritic mesopelagic zone biome', 'epeiric sea biome', 'marginal sea biome', 'mediterranean sea biome',
          'ocean biome', 'marine coral reef biome', 'marine subtidal rocky reef biome', 'marine black smoker biome',
          'marine white smoker biome', 'marine ultramafic hydrothermal vent biome',
          'marine basaltic hydrothermal vent biome', 'marine sponge reef biome', 'marine cold seep biome',
          'dilution basin mediterranean sea biome', 'woodland biome', 'shrubland biome', 'savanna biome',
          'tundra biome', 'mangrove biome', 'subtropical savanna biome', 'tropical savanna biome',
          'temperate savanna biome', 'flooded savanna biome', 'mediterranean woodland biome',
          'subtropical shrubland biome', 'tropical shrubland biome', 'temperate shrubland biome',
          'montane shrubland biome', 'mediterranean shrubland biome', 'xeric shrubland biome',
          'anthropogenic terrestrial biome', 'tropical woodland biome', 'temperate woodland biome',
          'subtropical woodland biome', 'montane savanna biome', 'mediterranean savanna biome', 'village biome',
          'rangeland biome', 'dense settlement biome', 'urban biome', 'freshwater lake biome', 'freshwater river biome',
          'polar biome', 'tropical marine coral reef biome', 'temperate marginal sea biome',
          'temperate mediterranean sea biome', 'marine upwelling biome', 'tropical marine upwelling biome',
          'temperate marine upwelling biome', 'tropical marginal sea biome', 'tidal mangrove shrubland',
          'alpine tundra biome', 'tropical biome', 'temperate biome', 'subtropical biome', 'mediterranean biome',
          'subpolar biome', 'alpine biome', 'montane biome', 'subalpine biome', 'arid biome']


def get_ngrams(text):
    return [''.join(gram) for gram in ngrams(text, 3)]


def calculate_cosine_similarity(query, corpus):
    vectorizer = TfidfVectorizer(analyzer=get_ngrams)
    tfidf_matrix = vectorizer.fit_transform(corpus)
    query_vector = vectorizer.transform([query])
    similarity_scores = cosine_similarity(query_vector, tfidf_matrix)[0]
    best_match_index = similarity_scores.argmax()
    best_match = corpus[best_match_index]
    return best_match


best_matches = []
for query in list_a:
    best_match = calculate_cosine_similarity(query, list_b)
    best_matches.append(best_match)

for query, best_match in zip(list_a, best_matches):
    print(f"Query: {query}")
    print(f"Best Match: {best_match}")
    print("---")



import csv
import click
from typing import List, Dict


@click.command()
@click.option('--in_file', default='assets/sheets-for-nmdc-submission-schema_import_slots_regardless.tsv',
              help='Full path to sheets-for-nmdc-submission-schema_import_slots_regardless.tsv including filename',
              type=str, required=True)
@click.option('--out_file', required=True, help='Output file path', type=str)
def cli(in_file: str, out_file: str):
    with open(in_file, "r") as f:
        reader = csv.DictReader(f, delimiter="\t")
        min_list = [
            {"slot": row["slot"], "destination class": dc}
            for row in reader
            for dc in row["destination class"].split("|")
        ]

    with open(out_file, "w") as f:
        writer = csv.DictWriter(f, fieldnames=['slot', 'destination class'], delimiter="\t")
        writer.writeheader()
        writer.writerows(min_list)


if __name__ == '__main__':
    cli()



import pprint
from typing import List

from nmdc_schema.get_nmdc_view import ViewGetter


class SchemaSlotsGetter:
    def get_schema_slots(self) -> List[str]:
        view_getter = ViewGetter()
        nmdc_view = view_getter.get_view()

        schema_slots = nmdc_view.all_slots()
        schema_slot_names = [v.name for k, v in schema_slots.items()]
        schema_slot_names.sort()
        return schema_slot_names




# SETUP

# get an ORCID if you don't have one: https://orcid.org/register
# log into https://data.microbiomedata.org or https://data.dev.microbiomedata.org with your ORCID
# get the NMDC session cookie value
#   firefox: open Firefox DevTools. On a Mac, you can use the command-option-c keystroke
#     click the storage tab at the top of the DevTools pane
#     click the "Cookies" button on the left
#     there should be an entry for https://data.microbiomedata.org or https://data.dev.microbiomedata.org, with a globe icon to the left
#     click the globe icon
#     find the session row
#     double click in the value column
#     copy the value with command-c
#     enter the session cookie value into the DATA_MICROBIOMEDATA_ORG_SESSION_COOKIE row in local/.env


import logging
import os
import pprint
import statistics
from dataclasses import dataclass
from typing import Optional, List, Dict, Any

import click
import click_log
import requests
from dotenv import load_dotenv
from linkml_runtime import SchemaView

logger = logging.getLogger(__name__)
click_log.basic_config(logger)


@dataclass
class SubmissionsSandbox:
    api_url: Optional[str] = None
    sample_metadata_by_slotname: Optional[Dict[str, List[List[str]]]] = None
    sample_metadata_by_title: Optional[Dict[str, List[List[str]]]] = None
    session_cookie: Optional[str] = None
    study_metadata: Optional[Dict[str, Any]] = None
    submission_schema_view: Optional[SchemaView] = None
    submission_slot_title_to_x: Optional[Dict[str, Dict[str, Any]]] = None
    url_suffix = "api/metadata_submission"

    schemas_dict = {
        # "mixs": "https://raw.githubusercontent.com/GenomicsStandardsConsortium/mixs/main/model/schema/mixs.yaml",
        "nmdc": "https://raw.githubusercontent.com/microbiomedata/nmdc-schema/main/src/schema/nmdc.yaml",
        "submission_schema": "https://raw.githubusercontent.com/microbiomedata/sheets_and_friends/main/artifacts/nmdc_submission_schema.yaml",
    }

    def get_one_submission_page_from_api(self, start, stop):
        logger.debug(f"session_cookie: {self.session_cookie}")
        logger.info(f"api_url: {self.api_url}")
        metadata_api_url = f"{self.api_url}{self.url_suffix}"
        logger.debug(f"whole_url: {metadata_api_url}")
        params = {"offset": start, "limit": stop}
        cookies = {"session": self.session_cookie}

        submission_response = requests.get(
            metadata_api_url, cookies=cookies, params=params
        )

        logger.debug(f"response received")

        submission_json = submission_response.json()

        # todo might want to check for response code

        logger.debug(f"conversion to JSON complete. About to get submission count.")

        submission_count = get_count_from_response(submission_json)
        logger.info(f"submission_count: {submission_count}")

        results = get_results_from_response(submission_json)
        results_pretty = pprint.pformat(results)
        logger.debug(f"results_pretty: {results_pretty}")

        (
            self.study_metadata,
            self.sample_metadata_by_title,
        ) = self.split_study_and_sample_metadata(results)

    def split_study_and_sample_metadata(self, results):
        study_metadata = {}
        sample_metadata_by_title = {}
        for result in results:
            result_id = result["id"]
            logger.debug(f"{result_id} started")
            logger.debug(f"result_id: {result_id}")
            if "metadata_submission" in result:
                metadata_submission = result["metadata_submission"]
                if "sampleData" in metadata_submission:
                    sample_data_lol = metadata_submission["sampleData"]
                    sample_data_lod = self.make_sample_data_dict(
                        sample_data_lol, result_id
                    )
                    sample_metadata_by_title[result_id] = sample_data_lod
            without_sample_data = self.del_sample_data_from_result(result)
            if without_sample_data:
                study_metadata[result_id] = without_sample_data
        return study_metadata, sample_metadata_by_title

    def make_sample_data_dict(
            self, sample_data: List[List[str]], result_id: str
    ) -> List[Dict[str, str]]:

        logger.debug(f"study_metadata: {self.study_metadata}")
        # todo may need to add or mint study id
        #   reuse the UUID that Kitware assigns to id,
        #   or some study identifier from the metadata_submission.multiOmicsForm or metadata_submission.studyForm paths?
        #   NMDC sample to study relationship expressed with sample_link?
        sample_data_row_count = len(sample_data)
        if result_id:
            if sample_data_row_count > 2:
                # todo make this test more flexible
                provided = set(sample_data[1])
                expected = set(self.submission_slot_title_to_x.keys())
                intersection = provided.intersection(expected)
                purity = len(intersection) / len(provided)
                # purity will almost always be a little lower than 1
                #   due to re-titling of slots in environmental package slot usages
                # but won't we have to resolve that eventually?
                if purity < 0.9:  # todo make this a parameter
                    logger.error(
                        f"study {result_id} doesn't seem to have the expected header rows: {sample_data[0:3]}"
                    )
                    # todo try to infer columns from template? what about versioning?
                    return [{}]
                else:
                    sample_data_headers = sample_data[1]
                    sample_data_body = sample_data[2:sample_data_row_count]
                    # todo the sample_data_headers values are term titles, not names
                    #  lookup with the latest version for now,
                    #  but work on better recording and reporting of schema version
                    body_list = []
                    for body_row in sample_data_body:
                        row_dict = dict(zip(sample_data_headers, body_row))
                        row_dict["sample_link"] = result_id
                        # todo this doesn't seem to work
                        #   would be handy for from_csv, if we want t support nmdc-schema v3 complaint samples
                        # row_dict["part_of"] = [result_id]
                        body_list.append(row_dict)

                    return body_list
            else:
                logger.error(
                    f"study {result_id} has less than three sample data rows (including the expected two headers)"
                )
                return [{}]
        else:
            logger.error(f"Empty result_id")
            return [{}]

    # @staticmethod
    def del_sample_data_from_result(self, result):
        if "metadata_submission" in result:
            metadata_submission = result["metadata_submission"]
            if "sampleData" in metadata_submission:
                for_counting = metadata_submission["sampleData"]
                del metadata_submission["sampleData"]
                if len(for_counting) > 2:
                    result["sample_data_total_rows"] = len(for_counting)
                    result["metadata_submission"] = metadata_submission
                    return result
                else:
                    result["sample_data_total_rows"] = 0
                    return result
        else:
            logger.error(f"no metadata_submission for {result['id']}")
            result["sample_data_total_rows"] = 0
            return result

    def get_submission_titles_and_names(self):
        submission_slots = self.submission_schema_view.schema.slots
        self.submission_slot_title_to_x = {
            v.title: {"title": v.title, "key": k, "alias": v.alias}
            for k, v in submission_slots.items()
            if v.title
        }
        self.submission_slot_key_to_x = {
            k: {"title": v.title, "key": k, "alias": v.alias}
            for k, v in submission_slots.items()
        }
        self.submission_slot_alias_to_x = {
            v.alias: {"title": v.title, "key": k, "alias": v.alias}
            for k, v in submission_slots.items()
            if v.alias
        }

    def view_setup(self, schemas_to_load: List[str] = None) -> None:
        logger.debug(f"will load schemas: {schemas_to_load}")
        if not schemas_to_load:
            logger.debug("will load the nmdc and submission schemas")
            schemas_to_load = ["nmdc", "submission_schema"]
        for schema_key in schemas_to_load:
            logger.info(f"loading schema {schema_key} into {schema_key}_view")
            setattr(self, f"{schema_key}_view", get_view(self.schemas_dict[schema_key]))


def get_session_cookie() -> str:
    return os.getenv("DATA_MICROBIOMEDATA_ORG_SESSION_COOKIE")


def get_count_from_response(api_response):
    logger.debug(f"looking for count in api_response")
    if "count" in api_response:
        return api_response["count"]
    else:
        logger.critical(
            f"api_response count = 0. {api_response} Do you need to refresh your session cookie in the .env file?")
        exit()


def get_results_from_response(api_response):
    return api_response["results"]


def get_view(schema_url):
    logger.info(f"creating a view of {schema_url}")
    try:
        view = SchemaView(schema_url)
        logger.info(f"confirming load of schema '{view.schema.name}'")
        return view
    except Exception as e:
        logger.critical(f"failed to load schema: {e}")
        exit()


def load_vars_from_env_file(env_file) -> None:
    load_dotenv(env_file)


@click.command()
@click_log.simple_verbosity_option(logger)
@click.option("--dotenv_file", type=click.Path(exists=True), default="local/.env")
@click.option("--page_start", default=0)
@click.option("--page_stop", default=999)
@click.option("--study_id", default="ad6a2a03-9229-463c-aeb6-29e1eb90b58f")
@click.option(
    "--data_portal_url",
    type=click.Choice(
        ["https://data.dev.microbiomedata.org/", "https://data.microbiomedata.org/"]
    ),
    default="https://data.microbiomedata.org/",
)
def cli(dotenv_file: str, data_portal_url: str, page_start: int, page_stop: int, study_id: str):
    load_vars_from_env_file(dotenv_file)

    submissions_sandbox = SubmissionsSandbox(api_url=data_portal_url, session_cookie=get_session_cookie())

    submissions_sandbox.view_setup()

    submissions_sandbox.get_submission_titles_and_names()

    submissions_sandbox.get_one_submission_page_from_api(start=page_start, stop=page_stop)

    pprint.pprint(submissions_sandbox.study_metadata[study_id])

    pprint.pprint(submissions_sandbox.sample_metadata_by_title[study_id])


if __name__ == "__main__":
    cli()



import csv
import logging
import os
import pprint
import re
from typing import List, Dict, Set, Any

import click
import click_log

logger = logging.getLogger(__name__)
click_log.basic_config(logger)


def replace_underscores(string):
    return string.replace("_", " ")


def split_string(string):
    return re.findall(r'\b\w+\b', string)


def lowercase_strings_in_list(string_list):
    return [s.lower() for s in string_list]


def trim_strings_in_list(string_list):
    return [s.strip() for s in string_list]


class MixsSlotTextMining:
    def __init__(self, label_file_mapping: Dict[str, str], tsv_file_root: str, dtm_tsv: str) -> None:
        self.label_file_mapping = label_file_mapping
        self.tsv_file_root = tsv_file_root
        self.dtm_tsv: str = os.path.join(tsv_file_root, dtm_tsv)
        self.token_set_dict: Dict[str, Set[str]] = {}
        self.token_list_dict: Dict[str, List[str]] = {}
        self.dtm_list: List[Dict[str, Any]] = []

    @staticmethod
    def tsv_file_to_dict_list(tsv_file):
        with open(tsv_file) as f:
            reader = csv.DictReader(f, delimiter="\t")
            return list(reader)

    def tsv_files_to_dict_list(self):
        for label, tsv_file in self.label_file_mapping.items():
            tsv_path = os.path.join(self.tsv_file_root, tsv_file)
            print(f"Reading {label} file: {tsv_path}")
            row_list = self.tsv_file_to_dict_list(tsv_path)
            for row in row_list:
                sc_name = row['Structured comment name']
                self.token_set_dict.setdefault(sc_name, set())
                for key in ('Structured comment name', 'Package item', 'Item (rdfs:label)'):
                    if key in row:
                        no_underscores = replace_underscores(row[key])
                        splitted = split_string(no_underscores)
                        lowercased = lowercase_strings_in_list(splitted)
                        trimmed = trim_strings_in_list(lowercased)
                        self.token_set_dict[sc_name].update(trimmed)

    def change_set_to_sorted_list(self):
        for key, value in self.token_set_dict.items():
            self.token_list_dict[key] = sorted(list(value))

    def make_document_term_matrix(self):
        self.dtm_list = []
        for slot_name, sorted_tokens in self.token_list_dict.items():
            row = {'slot_name': slot_name}
            for token in sorted_tokens:
                row[token] = 1
            self.dtm_list.append(row)
        # print(self.dtm_list)

    def write_dtm_tsv(self):
        tsv_cols = list(set().union(*(d.keys() for d in self.dtm_list)))
        tsv_cols.remove('slot_name')
        tsv_cols.sort()
        tsv_cols.insert(0, 'slot_name')

        with open(self.dtm_tsv, 'w') as f:
            writer = csv.DictWriter(f, fieldnames=tsv_cols, delimiter="\t")
            writer.writeheader()
            for row in self.dtm_list:
                writer.writerow(row)


@click.command()
@click_log.simple_verbosity_option(logger)
@click.option("--core_file", required=True)
@click.option("--packages_file", required=True)
@click.option("--output_file", required=True)
def cli(
        core_file,
        packages_file,
        output_file,
):
    MIxS_6_term_updates_tabs = {"core": core_file, "packages": packages_file}
    MIxS_6_term_updates_prefix = ""
    MIxS_6_term_updates_dtm_tsv = output_file
    mining_instance = MixsSlotTextMining(MIxS_6_term_updates_tabs, MIxS_6_term_updates_prefix,
                                         MIxS_6_term_updates_dtm_tsv)
    mining_instance.tsv_files_to_dict_list()
    mining_instance.change_set_to_sorted_list()
    mining_instance.make_document_term_matrix()
    mining_instance.write_dtm_tsv()


if __name__ == "__main__":
    cli()



from linkml_runtime import SchemaView
from typing import Dict, Optional, List
import difflib
import pandas as pd
import pprint
import requests

# import sqldf
from pandasql import sqldf

import click
import click_log
import logging

# using 'True' for True and None/'' for False

# todo: indicate borrowed
#  use see also s to indicate related mixs term?
#  indicate as_is, modified or inspired with something in addition to a dh_section (slot group?)

# todo from mixs annotation should also include version

pd.set_option("display.max_columns", None)

logger = logging.getLogger(__name__)
click_log.basic_config(logger)


@click.command()
@click_log.simple_verbosity_option(logger)
@click.option("--input_paths", "-i", multiple=True, required=True)
@click.option("--output_tsv", "-o", required=True)
def cli(input_paths, output_tsv):
    logger.info("Getting started")

    sr = SlotRoster()

    # todo how to do this programmatically
    sr.class_to_api_bit = {
        "biosample": "biosamples",
        "omics processing": "activities?filter=type%3Anmdc%3AOmicsProcessing",
    }

    sr.api_bit_to_class = {v: k for k, v in sr.class_to_api_bit.items()}

    # get from...
    # local filesystem?
    # w3id?
    # other URL?
    # submodule?
    for current_path in input_paths:
        schema_name = sr.add_view_from_file(schema_file=current_path)
        sr.add_schema_slot_names(sn=schema_name)
        sr.add_schema_class_slots(sn=schema_name)

    catted = pd.concat(sr.frame_dict.values(), ignore_index=True)

    query = """
    SELECT
        distinct class
    from
        catted
    wher
        "schema" = 'NMDC'
        and slot_schema = 'https://microbiomedata/schema/mixs'
        and class != '' and class is not null
    order by class
    """

    # where should this go?
    # PEP 8: E731 do not assign a lambda expression, use a def
    # pysqldf = lambda q: sqldf(q, globals())
    # mixs_slot_assoc_classes = sqldf.run(query)

    mixs_slot_assoc_classes = sqldf(query)

    mongo_frame_list = []
    for current_associated in mixs_slot_assoc_classes["class"]:
        api_bit = sr.class_to_api_bit[current_associated]
        inferred = sr.get_mongodb_collection_fields(api_bit)
        mongo_frame_list.append(inferred)

    mongo_frame = pd.concat(mongo_frame_list)

    catted["slot_no_ws"] = catted["slot"].str.replace(" ", "_")

    catted.rename(columns={"slot": "slot_raw", "slot_no_ws": "slot"}, inplace=True)

    recat = catted.merge(right=mongo_frame, how="outer", on=["class", "slot"])

    # todo are the recat slots used for biosamples?
    #  do they already have any data in mongodb?
    #  if yes/no, then just replace the old term with the new term?

    # ---

    # TODO should also be doing this with with nmdc_dh slots
    #  schema = nmdc_dh, from_schema = https://example.com/nmdc_dh
    #  would first want to remove anything that we know we added, like the EMSL and JGI slots...
    #    emsl_mixin
    #    jgi_mg_mixin
    #    jgi_mt_mixin
    #  BUT REVIEW expected new terms, wrt related MIxS terms? Do we have see_also s?
    #    samp_id_new_terms_mixin
    #    soil_mixs_inspired_mixin

    query = """
    SELECT
        distinct class, slot
    from
        recat
    where
        "schema" = 'NMDC'
        and slot_schema = 'https://microbiomedata/schema/mixs'
        and class is not NULL
        and class != ''
    """

    legacy_mixs_usage = sqldf(query)

    query = """
    SELECT
        distinct slot
    from
        recat
    where
        "schema" = 'MIxS'
        and slot_schema = 'http://w3id.org/mixs/terms';
    """

    now_available = sqldf(query)

    na_set = set(now_available["slot"])

    lost = list(set(legacy_mixs_usage["slot"]) - na_set)
    lost.sort()

    match_lod = []
    for current_lost in lost:
        matches = difflib.get_close_matches(current_lost, na_set, n=3, cutoff=0.9)
        i = 1
        if len(matches) == 0:
            match_lod.append({"slot": current_lost, "no_good_match": True})
        for current_match in matches:
            match_lod.append({"slot": current_lost, "match": current_match, "rank": i})
    match_frame = pd.DataFrame(match_lod)

    include_intersection = list(na_set.intersection(set(legacy_mixs_usage["slot"])))
    include_intersection.sort()

    merged = recat.merge(right=match_frame, how="left", on="slot")

    # temp = merged.loc[
    #     merged["slot_raw"].isin(list(match_frame["slot"]))
    #     & merged["schema"].eq("NMDC"),
    #     ["schema", "slot_raw", "match", "rank", "mongodb"],
    # ]
    # temp.drop_duplicates(inplace=True)

    merged.drop_duplicates(inplace=True)

    merged.to_csv(output_tsv, sep="\t", index=False)


class SlotRoster:
    def __init__(self):
        self.api_bit_to_class: Dict = {}
        self.frame_dict: Optional[Dict[str, pd.DataFrame]] = {}
        self.slots_dict: Optional[Dict[str, List[str]]] = {}
        self.view_dict: Optional[Dict[str, SchemaView]] = {}

    def add_view_from_file(self, schema_file: str) -> str:
        logger.info(f"opening {schema_file}")
        view = SchemaView(schema_file)
        sn = view.schema.name
        logger.info(f"adding {schema_file} as {sn}")
        self.view_dict[sn] = view
        return sn

    def add_schema_slot_names(self, sn: str, incl_imports: bool = True) -> None:
        logger.info(f"recording slots from {sn}")
        # had returned List[str]
        current_view = self.view_dict[sn]
        current_slot_obj = current_view.all_slots(imports=incl_imports)
        current_slot_names = [v.name for k, v in current_slot_obj.items()]
        current_slot_names.sort()
        self.slots_dict[sn] = current_slot_names

    def add_schema_class_slots(self, sn: str, incl_imports: bool = True) -> None:
        logger.info(f"recording class-slot relationships from {sn}")
        current_view = self.view_dict[sn]
        schema_classes = current_view.all_classes(imports=incl_imports)
        sc_names = list(schema_classes.keys())
        sc_names.sort()
        lod = []
        for current_class in sc_names:
            jit_class = current_view.get_class(current_class)
            # todo oops, wrap after 80 chars or so
            # logger.info(".", end="")
            # # got an unexpected keyword argument 'end'
            cis = current_view.class_induced_slots(current_class)
            for current_slot in cis:
                smbs = None
                if current_slot.mixin:
                    smbs = "True"
                sabs = None
                if current_slot.abstract:
                    sabs = "True"
                cmbs = None
                if jit_class.mixin:
                    cmbs = "True"
                cabs = None
                if jit_class.abstract:
                    cabs = "True"
                lod.append(
                    {
                        "schema": sn,
                        "class": current_class,
                        "slot": current_slot.name,
                        "slot_schema": current_slot.from_schema,
                        "source": current_slot.source,
                        "slot_title": current_slot.title,
                        "slot_group": current_slot.slot_group,
                        "slot_see_also": "|".join(current_slot.see_also),
                        "slot_mixin": smbs,
                        "slot_abstract": sabs,
                        "slot_is_a": current_slot.is_a,
                        "class_mixin": cmbs,
                        "class_abstract": cabs,
                        "class_is_a": jit_class.is_a,
                    }
                )
        # logger.info("")
        df = pd.DataFrame(lod)
        all_slots = set(self.slots_dict[sn])
        class_slots = set(df["slot"])
        no_class = list(all_slots - class_slots)
        no_class.sort()
        lod = []
        for current_nc in no_class:
            jit_slot = current_view.get_slot(current_nc)
            smbs = None
            if jit_slot.mixin:
                smbs = "True"
            sabs = None
            if jit_slot.abstract:
                sabs = "True"
            lod.append(
                {
                    "schema": sn,
                    "class": None,
                    "slot": current_nc,
                    "slot_schema": jit_slot.from_schema,
                    "source": jit_slot.source,
                    "slot_title": jit_slot.title,
                    "slot_group": jit_slot.slot_group,
                    "slot_see_also": "|".join(jit_slot.see_also),
                    "slot_mixin": smbs,
                    "slot_abstract": sabs,
                    "slot_is_a": jit_slot.is_a,
                }
            )
        df2 = pd.DataFrame(lod)
        df3 = pd.concat([df, df2])
        self.frame_dict[sn] = df3

    def get_mongodb_collection_fields(self, coll_type):
        current_page = 1
        cumulative_bs = []
        while True:
            url = f"https://api.dev.microbiomedata.org/{coll_type}"
            current_page = current_page + 1
            result = requests.get(url, params={"per_page": 200, "page": current_page})
            res_dict = result.json()
            res_res = res_dict["results"]
            res_len = len(res_res)
            logger.info(f"page {current_page}: {res_len} from {coll_type}")
            if res_len == 0:
                break
            cumulative_bs = cumulative_bs + res_res
        as_df = pd.DataFrame(cumulative_bs)
        inferred_slots = list(as_df.columns)
        lod = []
        for current_inferred in inferred_slots:
            lod.append(
                {
                    "class": self.api_bit_to_class[coll_type],
                    "slot": current_inferred,
                    "mongodb": "True",
                }
            )
        df = pd.DataFrame(lod)
        return df


if __name__ == "__main__":
    cli()



from linkml_runtime.dumpers import yaml_dumper

from nmdc_schema.nmdc_schema_accepting_legacy_ids import Biosample, ControlledIdentifiedTermValue, OntologyClass

bs = Biosample(id='xxx',
               part_of='gold:Gs0114663',
               env_medium=ControlledIdentifiedTermValue(
                   term=OntologyClass(id='ENVO:00010483', name='freshwater environment')),
               env_broad_scale=ControlledIdentifiedTermValue(
                   term=OntologyClass(id='ENVO:00010483', name='freshwater environment')),
               env_local_scale=ControlledIdentifiedTermValue(
                   term=OntologyClass(id='ENVO:00010483', name='freshwater environment')),
               )

print(yaml_dumper.dumps(bs))



import csv
import click


class ImportSlotsRegardless:
    mixs_slots_used_in_schema_tsv = "../assets/mixs_slots_used_in_schema.tsv"
    mixs_slots_used_in_schema = list()

    static_value_dict = {
        "source class": "soil MIMS",
        # "source file or URL": "https://raw.githubusercontent.com/GenomicsStandardsConsortium/mixs/main/model/schema/mixs.yaml",
        "source file or URL": "https://raw.githubusercontent.com/microbiomedata/mixs/1da849346a80b717810a02d7c8ed74a22bcd84de/model/schema/mixs.yaml",
        "slot": "",
        "section": "",
        "column order": "",
        "destination class": "placeholder_class"
    }

    def populate_mixs_slots_used_in_schema(self, input_file):
        with open(input_file, 'r') as file:
            self.mixs_slots_used_in_schema = [line.strip().split('\t')[0] for line in file]

    def make_import_slots_regardless_rows(self, input_file):
        self.populate_mixs_slots_used_in_schema(input_file)
        import_slots_regardless_rows = list()

        for slot in self.mixs_slots_used_in_schema:
            final_dict = self.static_value_dict.copy()
            final_dict["slot"] = slot
            import_slots_regardless_rows.append(final_dict)

        return import_slots_regardless_rows

    def write_import_slots_regardless_rows(self, output_file, input_file):
        fieldnames = list(self.static_value_dict.keys())
        slots_regardless_rows = self.make_import_slots_regardless_rows(input_file)
        with open(output_file, 'w', newline='') as output_file:
            writer = csv.DictWriter(output_file, fieldnames=fieldnames, delimiter='\t')
            writer.writeheader()
            for row in slots_regardless_rows:
                writer.writerow(row)


@click.command()
@click.option("--input_file", required=True, help="input file path from get_mixs_slots_used_in_schema")
@click.option("--output_file", required=True, help="output file path for sheets_and_friends")
def main(output_file, input_file):
    import_slots = ImportSlotsRegardless()
    import_slots.write_import_slots_regardless_rows(output_file, input_file)
    click.echo("Import slots regardless file generated successfully.")


if __name__ == '__main__':
    main()



from typing import List

from nmdc_schema.get_mixs_slots import MIxSSlotsGetter
from nmdc_schema.get_slots_from_view import SchemaSlotsGetter

import click


class IntersectionMixsSlotsGetter:
    mixs_term_getter: MIxSSlotsGetter = MIxSSlotsGetter()
    mixs_slotnames: List[str] = mixs_term_getter.get_unique_slot_names()
    slot_list = []

    def get_intersection_mixs_slots(self, slot_list_file) -> List[str]:
        print(f"slot_list_file: {slot_list_file}")
        with open(slot_list_file, 'r') as file:
            self.slot_list = [line.strip().split('\t')[0] for line in file]

        used_mixs_slots = list(set(self.mixs_slotnames) & set(self.slot_list))
        used_mixs_slots.sort()
        return used_mixs_slots


@click.command()
@click.option('--slot_list_file',
              '-s',
              type=click.Path(exists=True),
              required=True,
              help='Provide a single-column TSV file listing desired slots.')
@click.option('--output_file',
              '-o',
              type=click.File('w'),
              required=True,
              help='Your output will be written as a single-column TSV file.')
def main(slot_list_file, output_file):
    print(f"slot_list_file: {slot_list_file}")
    print(f"output_file: {output_file}")

    intersection_mixs_slot_getter = IntersectionMixsSlotsGetter()
    intersection_mixs_slots = intersection_mixs_slot_getter.get_intersection_mixs_slots(slot_list_file)

    with output_file:
        for slot in intersection_mixs_slots:
            output_file.write(f"{slot}\n")

    click.echo(f"The result has been written to {output_file.name}")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
A simple script to show how to retrive the jsonschema from the nmdc-schema library.
"""
if __name__ == "__main__":
    from nmdc_schema.validate_nmdc_json import get_nmdc_schema
    from pprint import pprint

    jsonschema = get_nmdc_schema()
    pprint(jsonschema)



from typing import List

from nmdc_schema.get_mixs_slots import MIxSSlotsGetter
from nmdc_schema.get_slots_from_view import SchemaSlotsGetter

import click


class UsedMixsSlotsGetter:
    mixs_term_getter: MIxSSlotsGetter = MIxSSlotsGetter()
    mixs_slotnames: List[str] = mixs_term_getter.get_unique_slot_names()

    schema_slots_getter: SchemaSlotsGetter = SchemaSlotsGetter()
    schema_slots: List[str] = schema_slots_getter.get_schema_slots()

    def get_used_mixs_slots(self) -> List[str]:
        used_mixs_slots = list(set(self.mixs_slotnames) & set(self.schema_slots))
        used_mixs_slots.sort()
        return used_mixs_slots


@click.command()
@click.option('--output_file',
              '-o',
              type=click.File('w'),
              required=True,
              help='Your output will be written as a single-column TSV file.')
def main(output_file):
    used_mixs_slot_getter = UsedMixsSlotsGetter()
    used_mixs_slots = used_mixs_slot_getter.get_used_mixs_slots()

    with output_file:
        for slot in used_mixs_slots:
            output_file.write(f"{slot}\n")

    click.echo(f"The result has been written to {output_file.name}")


if __name__ == '__main__':
    main()



import pprint

import pandas as pd
import yaml
from deepdiff import DeepDiff
from linkml_runtime import SchemaView

from strsimpy.cosine import Cosine

import click
import click_log
import logging

pd.set_option("display.max_columns", None)

logger = logging.getLogger(__name__)
click_log.basic_config(logger)


def y_file_to_dict(y_file_name):
    with open(y_file_name, "r") as stream:
        try:
            dict_from_yaml = yaml.safe_load(stream)
            return dict_from_yaml
        except yaml.YAMLError as exc:
            logger.warning(exc)


def get_slots_diff(deep_diff_obj, delta_type):
    # delta type is "added" or "removed"
    slots_diff = set()
    full_delta_type = f"dictionary_item_{delta_type}"
    if full_delta_type in deep_diff_obj:
        deep_diff_subset = deep_diff_obj[full_delta_type]
        for current_dds in deep_diff_subset:
            current_path = current_dds.path(output_format="list")
            if (
                len(current_path) == 3
                and current_path[0] == "slots"
                and current_path[2] == "name"
            ):
                slots_diff.add(current_path[1])
            if len(current_path) == 2 and current_path[0] == "slots":
                slots_diff.add(current_path[1])
        sd_list = list(slots_diff)
        sd_list.sort()
        return sd_list


def get_anno_diffs(
    deep_diff_obj, tsv_file_name, old_view, new_view, cosine_obj, incl_descrs=False
):
    lod = []
    if "values_changed" in deep_diff_obj:
        vc = deep_diff_obj["values_changed"]
        for current_vc in vc:
            current_path = current_vc.path(output_format="list")
            if (
                len(current_path) == 3
                and current_path[0] == "slots"
                and (current_path[2] != "description" or incl_descrs)
            ):
                to_append = {
                    "slot": current_path[1],
                    "attribute": current_path[2],
                }
                if current_path[2] == "description":
                    p0 = cosine_obj.get_profile(current_vc.t1)
                    p1 = cosine_obj.get_profile(current_vc.t2)
                    cos_dist_res = cosine_obj.similarity_profiles(p0, p1)
                    to_append["descr_dist"] = 1 - cos_dist_res
                    to_append["old_desc"] = current_vc.t1
                    to_append["new_desc"] = current_vc.t2
                else:
                    to_append["old_val"] = current_vc.t1
                    to_append["new_val"] = current_vc.t2
                # if current_path[2] == "range":
                #     old_element = old_view.get_element(current_vc.t1)
                #     to_append["old_range_type"] = type(old_element).class_name
                #     new_element = new_view.get_element(current_vc.t2)
                #     to_append["new_range_type"] = type(new_element).class_name
                lod.append(to_append)

        df = pd.DataFrame(lod)
        df.to_csv(tsv_file_name, sep="\t", index=False)


@click.command()
@click_log.simple_verbosity_option(logger)
@click.option("--include_descriptions", "-d", default=True)
@click.option("--shingle_size", "-o", default=2)
@click.option("--slot_diff_yaml_out", "-y", required=True)
@click.option("--anno_diff_tsv_out", "-t", required=True)
@click.option("--legacy_mixs_module_in", "-l", required=True)
@click.option("--current_mixs_module_in", "-c", required=True)
def cli(
    include_descriptions,
    shingle_size,
    slot_diff_yaml_out,
    anno_diff_tsv_out,
    legacy_mixs_module_in,
    current_mixs_module_in,
):
    logger.info("getting started")
    cosine_obj = Cosine(shingle_size)
    old_dict = y_file_to_dict(legacy_mixs_module_in)
    new_dict = y_file_to_dict(current_mixs_module_in)

    mixs_diff = DeepDiff(old_dict, new_dict, view="tree")

    # print(mixs_diff.keys())
    # # dict_keys(['dictionary_item_removed', 'dictionary_item_added', 'values_changed'])

    added_diff = get_slots_diff(mixs_diff, "added")

    removed_diff = get_slots_diff(mixs_diff, "removed")

    slots_diff_dict = {"added": added_diff, "removed": removed_diff}

    with open(slot_diff_yaml_out, "w") as outfile:
        yaml.dump(slots_diff_dict, outfile, default_flow_style=False)

    new_view = SchemaView(current_mixs_module_in)
    old_view = SchemaView(legacy_mixs_module_in)
    get_anno_diffs(
        mixs_diff,
        anno_diff_tsv_out,
        old_view,
        new_view,
        cosine_obj,
        include_descriptions,
    )


if __name__ == "__main__":
    cli()



import pprint
import urllib.request

from linkml_runtime.dumpers import json_dumper
from linkml_runtime.loaders import json_loader

from nmdc_schema.nmdc import Database

study_database_url = "https://raw.githubusercontent.com/microbiomedata/nmdc-schema/main/assets/test/data/study_test.json"
response = urllib.request.urlopen(study_database_url)
response_text = response.read().decode("utf-8")

study_database_instance = json_loader.load(source=response_text, target_class=Database)

study_database_dict = json_dumper.to_dict(study_database_instance)

pprint.pprint(study_database_dict)



import os
import sqlite3

import requests
import pandas as pd
import click
import requests_cache
from dotenv import load_dotenv


# https://en.wikipedia.org/wiki/USDA_soil_taxonomy


def remove_trailing_char(string, removal_char):
    if string.endswith(removal_char):
        return string.rstrip(removal_char)
    else:
        return string


def load_vars_from_env_file(env_file) -> None:
    load_dotenv(env_file)


def get_neon_token() -> str:
    return os.getenv("NEON_TOKEN")


@click.command()
@click.option('--envo-sqlite', default='local/envo.db', help='Path to ENVO SQLite database file')
@click.option("--dotenv-file", type=click.Path(exists=True), default="local/.env", help="Path to .env file")
@click.option('--soil-core-collection-csv-in', default='local/sls_soilCoreCollection.csv',
              help='Path to sls_soilCoreCollection.csv')
@click.option('--neon-site-envo-soil-mappings-tsv', default='local/neon-site-envo-soil-mappings.tsv',
              help='Path to output TSV file')
def generate_soil_order_envo_mappings(envo_sqlite, soil_core_collection_csv_in, dotenv_file,
                                      neon_site_envo_soil_mappings_tsv):
    load_vars_from_env_file(dotenv_file)

    url_base = "https://data.neonscience.org/api/v0/locations"

    token = get_neon_token()

    headers = {'X-API-Token': token}

    requests_cache.install_cache('neon_cache', backend='sqlite', expire_after=43200)  # 12 hours

    sls_soilCoreCollection_frame = pd.read_csv(soil_core_collection_csv_in, low_memory=False)
    named_location_counts = sls_soilCoreCollection_frame['namedLocation'].value_counts()
    # print(named_location_counts)

    named_locations_unique_list = named_location_counts.index.tolist()
    named_locations_unique_list.sort()

    location_count = len(named_locations_unique_list)

    soil_types_lod = []
    for idx, i in enumerate(named_locations_unique_list):
        print(f"{i} = {idx + 1} of {location_count}")

        soil_type_dict = {
            "envo_label_style": "soil",
            "site": i,
        }

        url = f"{url_base}/{i}"

        response = requests.get(url, headers=headers).json()

        if 'data' not in response:
            print("Skipping", i)
            continue

        location_properties_list = response['data']['locationProperties']
        for location_property in location_properties_list:

            if location_property['locationPropertyName'] == 'Value for Soil type order':
                neon_style = location_property['locationPropertyValue']
                envo_label_style = remove_trailing_char(neon_style.lower(), 's')
                soil_type_dict = {
                    "envo_label_style": envo_label_style,
                    "site": i,
                    "site_soil_type_order": location_property['locationPropertyValue'],
                }
                continue
        soil_types_lod.append(soil_type_dict)

    # pprint.pprint(soil_types_lod)

    site_soil_types_frame = pd.DataFrame(soil_types_lod)

    soil_types_frame = site_soil_types_frame[['envo_label_style', 'site_soil_type_order']].copy()
    soil_types_frame.drop_duplicates(inplace=True)

    envo_nlcd_alt_mappings_q = """
select
	ee.subject , s.value 
from
	entailed_edge ee
join statements s on
	s.subject = ee.subject
where
	ee."object" = "ENVO:00001998"
	and ee.predicate = "rdfs:subClassOf"
	and s.predicate = "rdfs:label"
    """

    conn = sqlite3.connect(envo_sqlite)
    c = conn.cursor()
    c.execute(envo_nlcd_alt_mappings_q)
    envo_soil_labels = c.fetchall()
    conn.close()

    # turn the sqlite results into a pandas dataframe
    envo_soil_labels_frame = pd.DataFrame(envo_soil_labels,
                                          columns=['envo_term', 'envo_label'])

    merged_df = pd.merge(soil_types_frame, envo_soil_labels_frame, left_on='envo_label_style', right_on='envo_label',
                         how='outer')

    merged_df.to_csv(neon_site_envo_soil_mappings_tsv, index=False, sep='\t')


if __name__ == '__main__':
    generate_soil_order_envo_mappings()



import csv
import pprint
import urllib.request

import csv
import io
import urllib.request
from typing import List


class MIxSSlotsGetter:
    # switch to retrieving from GSC' mixs.yaml
    def any_mixs_sheet_getter(self, url):
        response = urllib.request.urlopen(url)
        tsv_file = io.StringIO(response.read().decode('utf-8'))

        rows = []

        # Parse the file using DictReader
        reader = csv.DictReader(tsv_file, delimiter='\t')
        for row in reader:
            rows.append(row)

        return rows

    def extract_vals_from_rows(self, rows, var_name):
        slots = []
        for row in rows:
            if var_name in row and row[var_name] != '':
                slots.append(row[var_name])
        slots.sort()
        return slots

    def get_unique_slot_names(self,
                              var_name: str = "Structured comment name",
                              url_list: List[str] = (
                                      "https://docs.google.com/spreadsheets/d/1QDeeUcDqXes69Y2RjU2aWgOpCVWo5OVsBX9MKmMqi_o/export?gid=178015749&format=tsv",
                                      "https://docs.google.com/spreadsheets/d/1QDeeUcDqXes69Y2RjU2aWgOpCVWo5OVsBX9MKmMqi_o/export?gid=750683809&format=tsv"
                              )):
        slot_names = set()
        for url in url_list:
            parsed = self.any_mixs_sheet_getter(url)
            per_page_slots = self.extract_vals_from_rows(parsed, var_name)
            slot_names.update(per_page_slots)
        slot_names = list(slot_names)
        slot_names.sort()
        return slot_names



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Provides CLI to validate json files against the NMDC jsonschema.

This is now deprecated by either

- linkml-run-examples (which doesn't require it's input to be wrapped in a Database but doesn't exhaustively report errors.
- check-jsonschema (which does require it's input to be wrapped in a Database and does exhaustively report errors.)
"""

import json, jsonschema, click
from .nmdc_data import get_nmdc_jsonschema, get_nmdc_jsonschema_dict
from deprecated import deprecated


@deprecated(
    reason="functionality moved to nmdc_data.get_nmdc_jsonschema_dict()"
)
def get_nmdc_schema() -> dict:
    """
    Returns the nmdc.schema.json package data file as a dict.
    NOTE: This method is depricated. Should use nmdc_data.get_nmdc_jsonschema_dict()

    Returns
    -------
    dict
        Dict representation of the nmdc.schema.json package data file.
    """
    return get_nmdc_jsonschema_dict()


@deprecated(reason="functionality moved to nmdc_data.get_nmdc_jsonschema()")
def get_nmdc_schema_json() -> str:
    """
    Returns the nmdc.schema.json package data file json.
    NOTE: This method is depricated. Should use nmdc_data.get_nmdc_jsonschema()

    Returns
    -------
    str
        JSON string representation of the nmdc.schema.json package data file.
    """
    return get_nmdc_jsonschema()


def is_valid_json(json_file: str, database_set: str = "") -> bool:
    """
    Determines if the data in json_file conforms to the NMDC json schema.

    Parameters
    ----------
    json_file : str
        Path to the file containing json formatted data.
    database_set : str, default=""
        An optional top level database set (e.g, study_set, biosample_set) that contains the data.

    Returns
    -------
    bool
        True if data conforms to the NMDC json schema; False otherwise.
    """
    with open(json_file, "r") as fh:
        json_data = json.load(fh)

        database_set = database_set.strip()
        if len(database_set) > 0:
            if type(json_data) == type([]):
                json_data = {f"{database_set}": json_data}
            else:
                json_data = {f"{database_set}": [json_data]}
    try:
        jsonschema.validate(
            instance=json_data, schema=get_nmdc_jsonschema_dict()
        )
    except jsonschema.exceptions.ValidationError as err:
        print(err.message)
        return False

    return True


##### CLI interface #####
@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option(
    "--input",
    "-i",
    help="the path the file containing json formatted data.",
)
@click.option(
    "--database-set",
    "-set",
    default="",
    help="An optional top level database set (e.g, study_set, biosample_set) that contains the data.",
)
def cli(input: str, database_set: str):
    if is_valid_json(input, database_set):
        click.echo("%s: The JSON data is VALID for NMDC schema." % input)
    else:
        click.echo(
            "%s: The JSON data is ** NOT ** valid for NMDC schema." % input
        )


if __name__ == "__main__":
    cli()  # CLI interface



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Provides CLI to return versions of the NMDC Schema, meta model and pypi package."""

import pkg_resources, click
from .nmdc_data import get_nmdc_schema_definition


def get_nmdc_schema_package_version() -> str:
    """
    Returns the version of the nmdc_schema package.

    Returns
    -------
    str
        The version of the nmdc_schema package.
    """
    return pkg_resources.get_distribution("nmdc_schema").version


def get_nmdc_schema_version() -> str:
    """
    Returns the version specified in the nmdc.yaml file.

    Returns
    -------
    str
        The version in the nmdc.yaml file.
    """
    nmdc_schema = get_nmdc_schema_definition()
    return nmdc_schema.version


def get_nmdc_schema_metamodel_version() -> str:
    """
    Returns the metamodel version specified in the nmdc.yaml file.

    Returns
    -------
    str
        The metamodel version in the nmdc.yaml file.
    """
    nmdc_schema = get_nmdc_schema_definition()
    return nmdc_schema.metamodel_version


##### CLI interface #####
@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option(
    "--schema/--no-schema",
    "-s/-ns",
    default=True,
    show_default=True,
    help="specifies if nmdc schema version is printed",
)
@click.option(
    "--meta/--no-meta",
    "-m/-nm",
    default=False,
    show_default=True,
    help="specifies if nmdc schema metamodel version is printed",
)
@click.option(
    "--package/--no-package",
    "-pk/-npk",
    default=False,
    show_default=True,
    help="specifies if nmdc_schema package version is printed",
)
@click.option(
    "--all/--no-all",
    "-a/-na",
    default=False,
    show_default=True,
    help="specifies if all version information is printed",
)
def cli(schema, meta, package, all):
    if all == True:
        click.echo(get_nmdc_schema_version())
        click.echo(get_nmdc_schema_metamodel_version())
        click.echo(get_nmdc_schema_package_version())
    else:
        if meta == True:
            click.echo(get_nmdc_schema_metamodel_version())
        elif package == True:
            click.echo(get_nmdc_schema_package_version())
        else:
            click.echo(get_nmdc_schema_version())


if __name__ == "__main__":
    cli()  # CLI interface



import pkgutil

from linkml_runtime import SchemaView
from linkml_runtime.dumpers import json_dumper


class ViewGetter:
    def get_view(self) -> SchemaView:
        nmdc_schema_bytes = pkgutil.get_data("nmdc_schema", "nmdc_schema_merged.yaml")
        nmdc_schema_yaml_string = str(nmdc_schema_bytes, "utf-8")
        nmdc_view = SchemaView(nmdc_schema_yaml_string)
        return nmdc_view



import json
import os
import pprint
import re

import click
import click_log
from dotenv import load_dotenv
from linkml_runtime import SchemaView
from pymongo import MongoClient
from pymongo.errors import OperationFailure

import logging

logger = logging.getLogger(__name__)
click_log.basic_config(logger)

# make this a configuration file, or think of a better way to discover it in real time
#  the uniprot id mapping tool is manual and queued
uniprot_mnemonic_to_id = {
    "ALBU_BOVIN": "P02769",
    "ALBU_HUMAN": "P02768",
    "CTRA_BOVIN": "P00766",
    "K1C10_HUMAN": "P13645",
    "K1C9_HUMAN": "P35527",
    "K22E_HUMAN": "P35908",
    "K2C1_HUMAN": "P04264",
    "TRYP_BOVIN": "P00760",  # todo not sure about this
    "TRYP_PIG": "P00761",
}


# todo: in general, don't just change the id, also change referents

# def remove_carriage_returns_and_line_feeds(string):
#     return string.replace('\r', '').replace('\n', '')


def fix_curie(raw_curie):
    if ':' not in raw_curie:
        if raw_curie.count('_') > 0:
            modified_string = raw_curie.replace('_', ':', 1)
        else:
            modified_string = f"generic:{raw_curie}"
    else:
        curie_parts = raw_curie.split(':')
        if len(curie_parts) == 1 or curie_parts[1] == '' or curie_parts[1] == 'None' or curie_parts[1] == 'none':
            # if this is a term id, there may be an appropriate root term
            #   should be handled downstream
            # print(f"Warning: {raw_curie} is essentially prefix-only")
            pass
        modified_string = raw_curie
    # or urlencoding?
    tidied_string = re.sub(r'[^a-zA-Z0-9\-_.,:]', '', modified_string)
    return tidied_string


def access_database(env_file, mongo_db_name, mongo_host, mongo_port, admin_db):
    # Load MongoDB credentials from .env file
    load_dotenv(env_file)
    logger.info(f"loaded {env_file}")
    mongo_pw = os.getenv('SOURCE_MONGO_PASS')
    mongo_user = os.getenv('SOURCE_MONGO_USER')
    logger.info(f"{mongo_user = }")

    client = MongoClient(mongo_host,
                         port=mongo_port,
                         username=mongo_user,
                         password=mongo_pw,
                         authSource=admin_db,
                         authMechanism='SCRAM-SHA-256',  # todo should be an option
                         directConnection=True
                         )

    db = client[mongo_db_name]

    return db


def get_collection_names(mongo_db):
    collections = mongo_db.list_collection_names()
    return list(collections)


def set_arithmetic(set1, set2, set1_name='set 1 only', set2_name='set 2 only'):
    set1_only = set1 - set2
    set2_only = set2 - set1
    intersection = set1.intersection(set2)
    temp = {
        set1_name: set1_only,
        set2_name: set2_only,
        'intersection': intersection
    }
    return temp


def get_synonymous_collection_db_slots(mongo_db, schema_view: SchemaView, class_name='Database'):
    class_slots = schema_view.class_induced_slots(class_name)
    class_slot_names = [s.name for s in class_slots]
    class_slot_names.sort()

    # incomplete attempt to fiter out views
    collections = [coll['name'] for coll in mongo_db.list_collections() if 'viewOn' not in coll]

    collections.sort()

    # successful
    filtered_collections = []
    for collection in collections:
        try:
            # Check if the collection is a view
            if mongo_db[collection].count_documents({}) > 0:
                logger.info(f"Collection {collection} is accessible to this user and is not a view.")
                filtered_collections.append(collection)
            else:
                logger.info(f"Collection {collection} is a view.")
        except OperationFailure as e:
            if e.code == 13:  # Unauthorized error code
                logger.info(f"Collection {collection} is unauthorized for this user.")
                continue
            raise e

    synonymous_collection_names = set_arithmetic(set(class_slot_names), set(filtered_collections), set1_name='schema',
                                                 set2_name='mongo')

    return synonymous_collection_names


def get_doc_list(mongodb, collection_name, max_docs_per_coll):
    collection = mongodb[collection_name]

    if collection is None:
        logger.info(f"Could not find collection {collection_name}")

    documents = collection.find().limit(max_docs_per_coll)
    doc_list = list(documents)

    prob_key = "_id"
    for doc in doc_list:
        if prob_key in doc:
            del doc[prob_key]

    return doc_list


def get_collection_stats(mongo_db, collection_list):
    selected_stats = {}
    for collection_name in collection_list:
        for_selected_stats = {}

        collection_stats = mongo_db.command("collstats", collection_name)

        for_selected_stats["document_count"] = collection_stats["count"]
        for_selected_stats["size_in_bytes"] = collection_stats["size"]
        for_selected_stats["storageSize"] = collection_stats["storageSize"]

        selected_stats[collection_name] = for_selected_stats

    return selected_stats


@click.command()
@click_log.simple_verbosity_option(logger)
@click.option('--env-file', type=click.Path(), default='local/.env', help='Path to .env file')
@click.option('--mongo-db-name', default="nmdc", help='MongoDB database name')
@click.option('--selected-collections', multiple=True, help='MongoDB collection name')
@click.option('--root-class', default="Database", help='Schema class that corresponds to a Mongo Database')
@click.option('--mongo-host', default="localhost", help='MongoDB host name/address')
@click.option('--mongo-port', default=27777, help='MongoDB port')
@click.option('--admin-db', default="admin", help='MongoDB authentication source')
@click.option('--output-json', type=click.Path(), default='local/selected_mongodb_contents.json',
              help="Output directory. Exported file's name will be based on the collection name.")
@click.option('--output-json', type=click.Path(), help="Output file.")
@click.option('--schema-file', type=click.Path(), default='src/schema/nmdc.yaml',
              help='Path to root YAML file in the nmdc-schema')
@click.option('--max-docs-per-coll', default=100_000, help='Maximum number of documents to retrieve per collection')
@click.option('--non-nmdc-id-fixes/--no-non-nmdc-id-fixes', default=True,
              help="Apply repairs that don't alter ids for nmdc-schema instances")
@click.option('--curie-fix/--no-curie-fix', default=False,
              help='Remove end-of-line characters from nmdc-schema ids')
def export_to_yaml(selected_collections, env_file, mongo_db_name, mongo_host, mongo_port, admin_db, output_json,
                   schema_file, root_class, max_docs_per_coll, non_nmdc_id_fixes, curie_fix):
    db = access_database(env_file, mongo_db_name, mongo_host, mongo_port, admin_db)

    database = {}

    # no collections selected, so just summarize all of them
    if len(selected_collections) == 0:
        # just export all of them?
        # use a limit on document_count or size_in_bytes?
        # for now, just report the all (schema-relevant?) collection stats

        nmdc_view = SchemaView(schema_file)

        collections_to_check = get_synonymous_collection_db_slots(db, nmdc_view, root_class)

        logger.info(pprint.pformat(collections_to_check))

        mongo_collections = list(collections_to_check['intersection']) + list(collections_to_check['mongo'])

        mongo_collections.sort()

        collection_stats = get_collection_stats(db, mongo_collections)

        # # collection_stats = sorted(collection_stats.items(), key=lambda x: x[1]['size_in_bytes'])

        for coll_stat_k, coll_stat_v in collection_stats.items():
            if coll_stat_k in collections_to_check['intersection']:
                logger.info(f"Collection {coll_stat_k} is defined in the schema.")
                logger.info(pprint.pformat(coll_stat_v))
            # else:
            #     print(f"Collection {coll_stat_k} is not defined in the schema.")
            #     pprint.pprint(coll_stat_v)

    # now start exporting
    for selected_collection in selected_collections:
        # todo will need some error handling here
        collection_stats = get_collection_stats(db, [selected_collection])
        logger.info(f"Exporting collection {selected_collection}, with the following stats: {collection_stats}")
        doc_list = get_doc_list(db, selected_collection, max_docs_per_coll)

        # # # # REPAIRS # # # #

        # extract [] enclosed portion of  'samp_taxon_id': {'has_raw_value': 'lake water metagenome [NCBITaxon:1647806]'},
        #   and assign to samp_taxon_id.term.id
        if non_nmdc_id_fixes and selected_collection == 'biosample_set':
            for doc in doc_list:

                # REPAIR: extract Biosample samp_taxon_id term id from raw value if necessary
                #   these are external terms and don't require referential integrity withing the database
                taxon_slot = 'samp_taxon_id'
                if taxon_slot in doc and 'has_raw_value' in doc[taxon_slot]:
                    raw_value = doc[taxon_slot]['has_raw_value']
                    result = re.findall(r'\[(.*?)\]', raw_value)

                    if result:
                        extracted_text = result[0]
                        if 'term' not in doc[taxon_slot]:
                            logger.info(f"for {doc['id']}, setting the term id of {taxon_slot} to {extracted_text}")
                            doc[taxon_slot]['term'] = {'id': extracted_text}
                        else:
                            if 'id' not in doc[taxon_slot]['term']:
                                logger.info(f"for {doc['id']}, setting the term id of {taxon_slot} to {extracted_text}")
                                doc[taxon_slot]['term'] = {'id': extracted_text}
                            else:
                                logger.info(
                                    f"for {doc['id']}, replacing the previous {taxon_slot} term id of {doc[taxon_slot]['term']['id']} with {extracted_text}")
                                doc[taxon_slot]['term']['id'] = extracted_text
                    else:
                        # if there was a value in the taxon slot, wwe should report it
                        logger.info(f"for {doc['id']}, no term id found in {taxon_slot}")

                # REPAIR: extract Biosample host_taxid term id from raw value if necessary
                #   not using fix_curie
                #   these are external terms and don't require referential integrity withing the database
                taxon_slot = 'host_taxid'
                if taxon_slot in doc and 'has_raw_value' in doc[taxon_slot] and doc[taxon_slot]['has_raw_value'] == str(
                        int(
                            doc[taxon_slot]['has_raw_value'])):
                    extracted_text = doc[taxon_slot]['has_raw_value']
                    replacement_text = f"NCBITaxon:{extracted_text}"
                    if 'term' not in doc[taxon_slot]:
                        logger.info(f"for {doc['id']}, setting the term id of {taxon_slot} to {replacement_text}")
                        doc[taxon_slot]['term'] = {'id': replacement_text}
                    else:
                        if 'id' not in doc[taxon_slot]['term']:
                            logger.info(f"for {doc['id']}, setting the term id of {taxon_slot} to {replacement_text}")
                            doc[taxon_slot]['term'] = {'id': replacement_text}
                        else:
                            logger.info(
                                f"for {doc['id']}, replacing the previous {taxon_slot} term id of {doc[taxon_slot]['term']['id']} with {extracted_text}")
                            doc[taxon_slot]['term']['id'] = replacement_text
                else:
                    # if there was a value in the taxon slot, wwe should logger.info it out
                    logger.info(f"for {doc['id']}, no term id found in {taxon_slot}")

                # REPAIR: Biosample 'env_broad_scale', 'env_local_scale' and 'env_medium' ids must be id only,
                #   not label + value, like you would find in has_raw_value
                #   not using fix_curie
                #   these are external terms and don't require referential integrity withing the database
                # todo repetitive code
                for slot in ['env_broad_scale', 'env_local_scale', 'env_medium']:
                    logger.info(f"checking {slot} in {doc['id']}")
                    if slot in doc and 'term' in doc[slot] and 'id' in doc[slot]['term']:
                        raw_value = doc[slot]['term']['id']
                        logger.info(f"raw_value is {raw_value}")
                        # case 1: raw_value includes a label, and the id is enclosed in square brackets
                        result = re.findall(r'\[(.*?)\]', raw_value)
                        if result:
                            extracted_text = result[0]
                        else:
                            extracted_text = raw_value
                        # case 2: the raw text is delimited with an underscore instead of a colon
                        if "_" in extracted_text:
                            colonified = ':'.join(extracted_text.split("_"))
                        else:
                            colonified = extracted_text
                    if slot in doc:
                        if 'term' in doc[slot]:
                            if 'id' in doc[slot]['term']:
                                # carefully check these replacements. maybe we want to keep the original ids
                                logger.info(
                                    f"for {doc['id']}, replacing {slot} term id of {doc[slot]['term']['id']} with {colonified}")
                                doc[slot]['term']['id'] = colonified
                            else:
                                logger.info(f"for {doc['id']}, setting {slot} term id to {colonified}")
                                doc[slot]['term']['id'] = colonified
                        else:
                            logger.info(f"for {doc['id']}, setting {slot} term id to {colonified}")
                            doc[slot]['term'] = {'id': colonified}
                    else:
                        logger.info(f"checking {doc['id']} does not have a {slot}")

                # REPAIR: Biosample growth_facil.term.id: must be an id, not a string like 'field'
                #   not using fix_curie
                #   these are external terms and don't require referential integrity withing the database
                if 'growth_facil' in doc and 'term' in doc['growth_facil'] and 'id' in doc['growth_facil']['term']:
                    raw_value = doc['growth_facil']['term']['id']
                    if raw_value == 'field':
                        logger.info(
                            f"for {doc['id']}, replacing growth_facil term id of {raw_value} with ENVO:01000352")
                        doc['growth_facil'] = {
                            'has_raw_value': 'field',
                            'term': {
                                "name": "field",
                                'id': 'ENVO:01000352'

                            }
                        }

        # REPAIR: Biosample eliminate None value assertions for any slot from the root of any document in any collection
        #   not using fix_curie
        if non_nmdc_id_fixes:
            slots_to_del_if_none = [
                'asm_score',
                'binned_contig_num',
                'contig_bp',
                'contigs',
                'ctg_l50',
                'ctg_l90',
                'ctg_logsum',
                'ctg_max',
                'ctg_n50',
                'ctg_n90',
                'ctg_powsum',
                'gap_pct',
                'gc_avg',
                'gc_std',
                'input_base_count',
                'input_contig_num',
                'input_read_bases',
                'input_read_count',
                'low_depth_contig_num',
                'num_aligned_reads',
                'num_input_reads',
                'output_base_count',
                'output_read_bases',
                'output_read_count',
                'scaf_bp',
                'scaf_l50',
                'scaf_l90',
                'scaf_l_gt50k',
                'scaf_logsum',
                'scaf_max',
                'scaf_n50',
                'scaf_n90',
                'scaf_n_gt50k',
                'scaf_pct_gt50k',
                'scaf_powsum',
                'scaffolds',
                'too_short_contig_num',
                'unbinned_contig_num',
                "insdc_assembly_identifiers",
                "compression_type",
                "was_generated_by",
            ]
            for doc in doc_list:
                for slot in slots_to_del_if_none:
                    if slot in doc:
                        if not doc[slot]:
                            logger.info(f"for {doc['id']}, deleting {slot} because it is None")
                            del doc[slot]

        # REPAIR: Biosample eliminate None value assertions for any slot from the root of any document in any collection
        #  fix_curie() is a more aggressive fix than remove_carriage_returns_and_line_feeds()
        if curie_fix:
            for doc in doc_list:
                problem_slots = [
                    'has_input',
                    'has_output',
                    'id',
                    'part_of',
                ]
                for slot in problem_slots:
                    if slot in doc and type(doc[slot]) == list:
                        slotvals = doc[slot]
                        replacements = []
                        for raw_slotval in slotvals:
                            fixed_slotval = fix_curie(raw_slotval)
                            if fixed_slotval != raw_slotval:
                                logger.info(
                                    f"{selected_collection} {doc['id']} {slot} [{raw_slotval}] -> {fixed_slotval}")
                            replacements.append(fixed_slotval)
                        doc[slot] = replacements
                    elif slot in doc:
                        raw_slotval = doc[slot]
                        fixed_slotval = fix_curie(raw_slotval)
                        if raw_slotval != fixed_slotval:
                            logger.info(
                                f"{selected_collection} {doc['id']} {slot} {raw_slotval} -> {fixed_slotval}")
                        doc[slot] = fixed_slotval

        # REPAIR: metabolite quantification alternative identifiers must use prefix:local CURIEs
        #   these are external terms and don't require referential integrity withing the database
        # why are the objects of nmdc:alternative_identifiers typed like "cas:3685-26-5"^^xsd:anyURI ?
        if non_nmdc_id_fixes and selected_collection == 'metabolomics_analysis_activity_set':
            for doc in doc_list:
                id_val = doc['id']
                if 'has_metabolite_quantifications' in doc:
                    accepted_mqs = []
                    for mq in doc['has_metabolite_quantifications']:
                        if 'alternative_identifiers' in mq:
                            raw_ais = mq['alternative_identifiers']
                            accepted_ais = []
                            for ai in raw_ais:
                                alt_id_parts = ai.split(":")
                                if alt_id_parts[1] in ['', 'None']:
                                    # verbose mode only
                                    # print(
                                    #     f"{selected_collection} {id_val} has_metabolite_quantifications.alternative_identifiers has un-redeemable metabolite identifier: [{ai}]")
                                    pass
                                else:
                                    # not even using fix_curie yet
                                    raw_slotval = ai
                                    fixed_slotval = fix_curie(raw_slotval)
                                    if raw_slotval != fixed_slotval:
                                        logger.info(
                                            f"{selected_collection} {id_val} has_metabolite_quantifications.alternative_identifiers [{raw_slotval}] -> {fixed_slotval}")
                                    accepted_ais.append(fixed_slotval)
                            mq['alternative_identifiers'] = accepted_ais
                        accepted_mqs.append(mq)
                    doc['has_metabolite_quantifications'] = accepted_mqs

        # REPAIR: slots that refer to instances of another class,
        #   even if that class is never instantiated, must use prefix:local CURIEs
        def convert_contam_string_to_curie(value):
            # if ":" not in value and value.startswith("Contaminant"):
            parts = value.split("_")
            if len(parts) == 3:  # Change to 3 if you want exactly three parts
                local_mnenomic_id = '_'.join(parts[1:3])
                local_unprot_id = uniprot_mnemonic_to_id[local_mnenomic_id]
                curie = "UniProtKB:" + local_unprot_id
                return curie
            return None

        if non_nmdc_id_fixes and selected_collection == 'metaproteomics_analysis_activity_set':
            for doc in doc_list:
                if 'has_peptide_quantifications' in doc:
                    repaired_pqs = []
                    for pq in doc['has_peptide_quantifications']:
                        if 'all_proteins' in pq:
                            repaired_aps = []
                            for ap in pq['all_proteins']:
                                if ":" not in ap:
                                    if ap.startswith("Contaminant_"):
                                        as_up_contam = convert_contam_string_to_curie(ap)
                                        # log the original collection, id, and value plus the repaired value
                                        if as_up_contam != ap:
                                            logger.info(
                                                f"{selected_collection} {doc['id']} has_peptide_quantifications.all_proteins [{ap}] -> {as_up_contam}")
                                        repaired_aps.append(as_up_contam)
                                    else:
                                        repaired_aps.append(ap)
                                pq['all_proteins'] = repaired_aps
                        if 'best_protein' in pq:
                            bp = pq['best_protein']
                            if ":" not in bp:
                                if bp.startswith("Contaminant_"):
                                    as_up_contam = convert_contam_string_to_curie(bp)
                                    pq['best_protein'] = as_up_contam
                                    logger.info(
                                        f"{selected_collection} {doc['id']} has_peptide_quantifications.best_proteins {bp} -> {as_up_contam}")
                            else:
                                pq['best_protein'] = bp
                        repaired_pqs.append(pq)
                    doc['has_peptide_quantifications'] = repaired_pqs

        database[selected_collection] = doc_list

    # # save documents to JSON file with pretty formatting/indentation
    # # avoids complaints about 64-bit numbers in YAML?
    # output_json = output_yaml.replace(".yaml", ".json")
    if output_json:
        with open(output_json, 'w') as f:
            json.dump(database, f, indent=2)


if __name__ == '__main__':
    export_to_yaml()



import re

import yaml
import sqlite3
import pandas as pd
import click


# Hugh has already provided a partial, highly curated mapping
# Harry is looking into oaklib/ontogpt solutions


# nlcd metadata is included in each release from the
#   Multi-Resolution Land Characteristics (MRLC) Consortium
# this page highlights the MRLC land cover releases for the Continental US
#   https://www.mrlc.gov/data?f%5B0%5D=category%3ALand%20Cover&f%5B1%5D=region%3Aconus
# make sure we are using the same version an NEON
# Here I will illustrate the use of "NLCD 2019 Land Cover (CONUS)"
# click on the faint "More" link in the lower right-hand corner of the  "NLCD 2019 Land Cover (CONUS)" tile
# click on the "Metadata" link
# that will take you to an XML document wth sections like

# what's edom?
# are there any other authorities on this subject?
# envthes?

# <attrdomv>
#   <edom>
#     <edomv>43</edomv>
#       <edomvd>Mixed Forest - Areas dominated by trees generally greater than 5 meters tall, and greater than 20% of total vegetation cover. Neither deciduous nor evergreen species are greater than 75 percent of total tree cover.</edomvd>
#       <edomvds>NLCD Legend Land Cover Class Descriptions</edomvds>
#   </edom>
# </attrdomv>

# have not been able to parse that file with newbie style XML parsing
# so start by downloading and converting to YAML with local/nlcd_2019_land_cover_l48_20210604.yaml
#   in project.Makefile

# ABBY_002.basePlot.bgc
# namedLocation in sls_soilCoreCollection.csv -> location lookup
#   https://data.neonscience.org/api/v0/locations/ABBY_002.basePlot.bgc
# locationProperties
#       {
#         "locationPropertyName": "Value for Soil type order",
#         "locationPropertyValue": "Andisols"
#       },


@click.command()
@click.option('--nlcd-yaml', default='local/nlcd_2019_land_cover_l48_20210604.yaml', help='Path to NLCD YAML file')
@click.option('--envo-sqlite', default='local/envo.db', help='Path to ENVO SQLite database file')
@click.option('--neon-nlcd-envo-mappings-tsv', default='local/neon-nlcd-envo-mappings.tsv',
              help='Path to output TSV file')
def generate_nlcd_envo_mappings(nlcd_yaml, envo_sqlite, neon_nlcd_envo_mappings_tsv):
    with open(nlcd_yaml, "r") as stream:
        try:
            nlcd_dict = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)

    detailed_attrs = nlcd_dict['metadata']['eainfo']['detailed']['attr']

    edoms = []
    for detailed_attr in detailed_attrs:
        if detailed_attr['attrdefs'] == 'NLCD Legend Land Cover Class Descriptions':
            edoms = detailed_attr['attrdomv']
            break

    edom_lod = []
    for edom in edoms:
        extracted_portion = edom['edom']['edomvd'].split(' -')[0]

        # Remove whitespace and punctuation, and split into words
        words = re.findall(r'\w+', extracted_portion)

        # Convert to lower camel case
        lower_camel_case = words[0].lower() + ''.join(word.capitalize() for word in words[1:])

        envo_style_alt_id = f"NLCD:{edom['edom']['edomv']}"

        edom_dict = {
            "envo_style_alt_id": envo_style_alt_id,
            "full_desc": edom['edom']['edomvd'],
            "num_key": edom['edom']['edomv'],
            'extractedPortion': extracted_portion,
            'lowerCamelCase': lower_camel_case,
        }
        edom_lod.append(edom_dict)

    # Define the field names based on the keys in the dictionaries
    fieldnames = edom_lod[0].keys()

    # convert that list of dictionaries into a pandas dataframe
    edom_df = pd.DataFrame(edom_lod)

    envo_nlcd_alt_mappings_q = """
    select
        s1.subject, s1.value, s2.value
    from
        statements s1
    join statements s2 on s1.subject = s2.subject 
    where
        s1.predicate  = 'oio:hasAlternativeId' and s1.value like 'NLCD:%'
        and s2.predicate = "rdfs:label"
    """

    conn = sqlite3.connect(envo_sqlite)
    c = conn.cursor()
    c.execute(envo_nlcd_alt_mappings_q)
    envo_nlcd_alt_mappings = c.fetchall()
    conn.close()

    # turn the sqlite results into a pandas dataframe
    envo_nlcd_alt_mappings_df = pd.DataFrame(envo_nlcd_alt_mappings,
                                             columns=['envo_term', 'nlcd_mapping', 'envo_label'])

    merged_df = pd.merge(edom_df, envo_nlcd_alt_mappings_df, left_on='envo_style_alt_id', right_on='nlcd_mapping',
                         how='outer')

    merged_df.to_csv(neon_nlcd_envo_mappings_tsv, index=False, sep='\t')


if __name__ == '__main__':
    generate_nlcd_envo_mappings()
